import 'package:example/backend/services/account/account_service.dart';
import 'package:example/injector.dart';
import 'package:example/storage/count_storage.dart';
import 'package:get/get.dart';
import 'package:injectable/injectable.dart';
import 'package:suga_core/suga_core.dart';

@injectable
class CountPageModel extends BaseViewModel {
  RxInt count = 10.obs;
  RxString description = 'Your storage is 10'.obs;

  String email;
  String code;

  @override
  void initState() {
    super.initState();
    sendOTP();
  }

  void increase() {
    ++count;
    injector<CountStorage>().set(count.value);
    description.value = "Your storage is ${injector<CountStorage>().get()}";
  }

  Future<void> sendOTP() async {
    Map<String, dynamic> params = {};
    params["email"] = email;
    injector.get<AccountService>().sendOtp(params).then((value) {
      print(value);
    }, onError: ((error) {
      if (error is HttpError) {
        print(error.message);
      } else {
        print(error.toString());
      }
    }));
  }

  Future<void> login() async {
    await injector<Oauth2Manager>().getCredential(email, code);
    String accessToken = await injector<Oauth2Manager>().getAccessToken();
    print(accessToken);
  }
}
