import 'dart:io';

import 'package:dio/dio.dart';
import 'package:suga_core/src/http/http_client_config.dart';
import 'package:suga_core/src/http/http_error.dart';

class HttpClient {
  final Dio dio = Dio();

  final HttpClientConfig config;

  HttpClient(this.config) {
    dio.options = config.dioOptions;

    if (config.logger != null) {
      if (config.verbose) {
        dio.interceptors.add(LogInterceptor(logPrint: config.logger.v, requestBody: true, responseBody: true));
      } else {
        dio.interceptors.add(LogInterceptor(logPrint: config.logger.d));
      }
    }

    dio.interceptors.add(InterceptorsWrapper(onRequest: _onRequest, onError: _onError));

    if (config.extraInterceptors != null) {
      dio.interceptors.addAll(config.extraInterceptors);
    }
  }

  Future<RequestOptions> _onRequest(RequestOptions option) async {
    option.headers["Accept"] = "application/json";
    String accessToken = await config.oauth2Manager?.getAccessToken();
    if (accessToken != null) {
      option.headers["Authorization"] = "Bearer $accessToken";
    }

    if (config.extraHeaders != null) {
      option.headers.addAll(config.extraHeaders);
    }

    return option;
  }

  HttpError _onError(DioError e) {
    final HttpError httpError = HttpError(e);

    if (httpError.response?.statusCode == HttpStatus.unauthorized) {
      config.logger?.w("Http Unauthorized Response");
      if (config.unauthorizedResponseHandler != null) {
        config.unauthorizedResponseHandler();
      }
    }

    return httpError;
  }
}
