import 'package:oauth2/oauth2.dart';
import 'package:suga_core/src/oauth2/oauth2_manager_config.dart';
import 'package:synchronized/synchronized.dart';

class Oauth2Manager {
  final Oauth2ManagerConfig config;

  Oauth2Manager(this.config);

  Future<Credentials> getCredential(String username, String password) async {
    config?.logger?.d("Logging in - username $username, password $password");

    final Client client = await resourceOwnerPasswordGrant(config.authorizationEndpoint, username, password, identifier: config?.identifier, secret: config?.secret, basicAuth: config?.basicAuth);

    final Credentials credentials = client.credentials;
    config?.credentialStorage?.set(credentials);
    config?.logger?.d("Logged in - credentials ${credentials.toJson()}");
    return credentials;
  }

  Future<Credentials> _refresh({Credentials credentials}) async {
    credentials ??= config?.credentialStorage?.get();

    config?.logger?.d("Refreshing - credentials ${credentials.toJson()}");

    if (credentials == null) {
      throw StateError("Credentials must be not null");
    }

    final newCredentials = await credentials.refresh(identifier: config?.identifier, secret: config?.secret, basicAuth: config.basicAuth);
    config?.credentialStorage?.set(newCredentials);

    config?.logger?.d("Refreshed - credentials ${newCredentials.toJson()}");
    return newCredentials;
  }

  Future<bool> isLoggedIn() async {
    final Credentials credentials = config?.credentialStorage?.get();
    return credentials != null && !credentials.isExpired;
  }

  Future<String> getAccessToken() async {
    Credentials credentials = config?.credentialStorage?.get();

    if (credentials == null) {
      return null;
    }

    if (credentials.isExpired) {
      await Lock().synchronized(() async {
        if (credentials.isExpired) {
          credentials = await _refresh(credentials: credentials);
        }
      });
      // ignore: invariant_booleans
      if (credentials == null) {
        return null;
      }
    }

    return credentials.accessToken;
  }
}
