import 'package:flutter/widgets.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:suga_core/src/http/http_error.dart';
import 'package:synchronized/synchronized.dart';

abstract class BaseViewModel {
  dynamic _error;

  String get errorMessage {
    if (_error == null) {
      return null;
    }

    if (_error is HttpError) {
      final HttpError httpError = _error as HttpError;
      return httpError.message;
    }

    return _error.toString();
  }

  void initState() {}

  void disposeState() {}

  @protected
  Future<bool> call(dynamic Function() handler, {bool toastOnError = false, bool background = false}) async {
    return await Lock().synchronized(() async {
      try {
        _error = null;
        if (!background) {
          await EasyLoading.show(status: "Please wait");
        }
        final result = handler();
        if (result is Future) {
          await result;
        }
        if (!background) {
          await EasyLoading.dismiss();
        }
        return true;
      } catch (e) {
        _error = e;
        if (!background) {
          await EasyLoading.dismiss();
        }
        if (toastOnError) {
          await Fluttertoast.showToast(msg: errorMessage);
        }
        return false;
      }
    });
  }
}
